/*
Test sketch for CANADUINO® PLC-100 V2
The RTC is synchronized with the host time during upload.
The digital inputs 1 to 4 turn on the relays 1 to 4.
The analog outputs 1 to 4 follow the analog inputs 1 to 4.
A value >490 on any analog input turns on relay 5.
A value >980 on any analog input turns on relay 6.
The serial monitor outputs a time stamp, followed by the
status of every digital and analog input and output,
and a reading of the analog inputs 1 to 4, every 200ms.  
*/


#include <Wire.h>
#include <RTClib.h> // Include the RTC library

RTC_DS1307 rtc; // Create an instance of the DS1307 RTC object

void setup() {
  Serial.begin(9600);
  Wire.begin();  // Use default I2C pins for the ATmega328P
  
  if (!rtc.begin()) {
    Serial.println("Couldn't find RTC");
    while (1);
  }

  if (!rtc.isrunning()) {
    Serial.println("RTC is NOT running!");
    // Set the RTC to the date & time this sketch was compiled
    rtc.adjust(DateTime(F(__DATE__), F(__TIME__)));
  }
 
  // Configure digital pins as inputs
  pinMode(7, INPUT);  // Digital Input 1
  pinMode(8, INPUT);  // Digital Input 2
  pinMode(12, INPUT); // Digital Input 3
  pinMode(13, INPUT); // Digital Input 4

  // Configure pins for relay control
  pinMode(2, OUTPUT); // Relay Output 1
  pinMode(3, OUTPUT); // Relay Output 2
  pinMode(4, OUTPUT); // Relay Output 3
  pinMode(5, OUTPUT); // Relay Output 4
  pinMode(A2, OUTPUT); // Relay Output 5
  pinMode(A3, OUTPUT); // Relay Output 6
}

void loop() {
  DateTime now = rtc.now(); // Get current time from RTC

  // Read digital inputs and activate corresponding relays
  if (digitalRead(7) == HIGH) {
    digitalWrite(2, HIGH); // Relay 1
  } else {
    digitalWrite(2, LOW);
  }
  if (digitalRead(8) == HIGH) {
    digitalWrite(3, HIGH); // Relay 2
  } else {
    digitalWrite(3, LOW);
  }
  if (digitalRead(12) == HIGH) {
    digitalWrite(4, HIGH); // Relay 3
  } else {
    digitalWrite(4, LOW);
  }
  if (digitalRead(13) == HIGH) {
    digitalWrite(5, HIGH); // Relay 4
  } else {
    digitalWrite(5, LOW);
  }

  // Read analog inputs and activate relays 5 and 6 accordingly
  int analogValue1 = analogRead(A0);
  int analogValue2 = analogRead(A1);
  int analogValue3 = analogRead(A6); // A6 can only be used as analog input
  int analogValue4 = analogRead(A7); // A7 can only be used as analog input

  if (analogValue1 >= 490 || analogValue2 >= 490 || analogValue3 >= 490 || analogValue4 >= 490) {
    digitalWrite(A2, HIGH); // Relay 5
  } else {
    digitalWrite(A2, LOW);
  }

  if (analogValue1 >= 980 || analogValue2 >= 980 || analogValue3 >= 980 || analogValue4 >= 980) {
    digitalWrite(A3, HIGH); // Relay 6
  } else {
    digitalWrite(A3, LOW);
  }

  // PWM outputs based on analog input values
  analogWrite(6, calculatePWM(analogValue1)); // PWM Output 1
  analogWrite(9, calculatePWM(analogValue2)); // PWM Output 2
  analogWrite(10, calculatePWM(analogValue3)); // PWM Output 3
  analogWrite(11, calculatePWM(analogValue4)); // PWM Output 4

  // Enhanced debug output with actual time/date stamp
  char timestamp[20];
  sprintf(timestamp, "%04d-%02d-%02d %02d:%02d:%02d", now.year(), now.month(), now.day(), now.hour(), now.minute(), now.second());

  Serial.print(timestamp); Serial.print(" | ");
  Serial.print(digitalRead(7)); Serial.print("  ");
  Serial.print(digitalRead(8)); Serial.print("  ");
  Serial.print(digitalRead(12)); Serial.print("  ");
  Serial.print(digitalRead(13)); Serial.print("  ");
  Serial.print(digitalRead(2)); Serial.print("  ");
  Serial.print(digitalRead(3)); Serial.print("  ");
  Serial.print(digitalRead(4)); Serial.print("  ");
  Serial.print(digitalRead(5)); Serial.print("  ");
  Serial.print(analogRead(A0)); Serial.print("  ");
  Serial.print(analogRead(A1)); Serial.print("  ");
  Serial.print(analogRead(A6)); Serial.print("  ");
  Serial.println(analogRead(A7));

  delay(200); // Adjust delay as needed
}

int calculatePWM(int analogValue) {
  // Calculate PWM duty cycle using linear interpolation
  int pwm = map(analogValue, 0, 1023, 0, 255); // Adjust the range for 10-bit ADC of ATmega328P

  // Ensure PWM duty cycle does not exceed 255
  return min(pwm, 255);
}
